@@ -111,10 +111,10 @@ open_from_uri (GnomeVFSDirectoryHandle **handle,
 }
 
 static GnomeVFSResult
-open (GnomeVFSDirectoryHandle **handle,
-      const gchar *text_uri,
-      GnomeVFSFileInfoOptions options,
-      GnomeVFSContext *context)
+do_open (GnomeVFSDirectoryHandle **handle,
+	 const gchar *text_uri,
+	 GnomeVFSFileInfoOptions options,
+	 GnomeVFSContext *context)
 {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
@@ -154,7 +154,7 @@ gnome_vfs_directory_open (GnomeVFSDirectoryHandle **handle,
 	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
-	return open (handle, text_uri, options, NULL);
+	return do_open (handle, text_uri, options, NULL);
 }
 
 /**
@@ -483,16 +483,16 @@ gnome_vfs_directory_visit_uri (GnomeVFSURI *uri,
 
 /**
  * gnome_vfs_directory_visit:
- * @uri: URI to start from
+ * @text_uri: URI to start from
  * @info_options: Options specifying what kind of file information must be
  * retrieved
  * @visit_options: Options specifying the type of visit
  * @callback: Callback to be called for every visited file
  * @data: Data to be passed to @callback at each iteration
  * 
- * Visit @uri, retrieving information as specified by @info_options.
+ * Visit @text_uri, retrieving information as specified by @info_options.
  * 
- * Return value: 
+ * Return value:  a GnomeVFSResult indication the success of the operation 
  **/
 GnomeVFSResult
 gnome_vfs_directory_visit (const gchar *text_uri,
@@ -507,6 +507,9 @@ gnome_vfs_directory_visit (const gchar *text_uri,
 	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
+	if (uri == NULL) {
+		return GNOME_VFS_ERROR_INVALID_URI;
+	}
 
 	result = directory_visit_internal (uri, NULL, NULL,
 					   info_options,
