@@ -214,7 +214,10 @@ split_toplevel_uri (const gchar *path, guint path_len,
 		/* *cur == ':' or '@' and string contains a @ before a / */
 
 		if (uri_strlen_to (cur_tok_start, cur) > 0) {
-			*user_return = uri_strdup_to (cur_tok_start,cur);
+			char *tmp;
+			tmp = uri_strdup_to (cur_tok_start,cur);
+			*user_return = gnome_vfs_unescape_string (tmp, NULL);
+			g_free (tmp);
 		}
 
 		if (*cur == ':') {
@@ -226,7 +229,10 @@ split_toplevel_uri (const gchar *path, guint path_len,
 				success = FALSE;
 				goto done;
 			} else if (uri_strlen_to (cur_tok_start, cur) > 0) {
-				*password_return = uri_strdup_to (cur_tok_start,cur);
+				char *tmp;
+				tmp = uri_strdup_to (cur_tok_start,cur);
+				*password_return = gnome_vfs_unescape_string (tmp, NULL);
+				g_free (tmp);
 			}
 		}
 
@@ -384,7 +390,9 @@ set_uri_element (GnomeVFSURI *uri,
 	 * meaning.
 	 */
 	if ( ! (strcmp (uri->method_string, "http") == 0 
-	        || strcmp (uri->method_string, "eazel-services") == 0
+	        || strcmp (uri->method_string, "https") == 0
+		|| strcmp (uri->method_string, "dav") == 0
+		|| strcmp (uri->method_string, "davs") == 0
 	        || strcmp (uri->method_string, "ghelp") == 0
 	        || strcmp (uri->method_string, "gnome-help") == 0
 	        || strcmp (uri->method_string, "help") == 0
@@ -410,7 +418,12 @@ get_method_string (const gchar *substring, gchar **method_string)
 	     p++)
 		;
 
-	if (*p == ':') {
+	if (*p == ':'
+#ifdef G_OS_WIN32
+	              &&
+	    !(p == substring + 1 && g_ascii_isalpha (*substring))
+#endif
+								 ) {
 		/* Found toplevel method specification.  */
 		method = g_strndup (substring, p - substring);
 		*method_string = g_ascii_strdown (method, -1);
@@ -489,6 +502,34 @@ parse_uri_substring (const gchar *substring, GnomeVFSURI *parent)
 GnomeVFSURI *
 gnome_vfs_uri_new (const gchar *text_uri)
 {
+#ifdef G_OS_WIN32
+	gchar *method_string;
+	const gchar *rest;
+
+	rest = get_method_string (text_uri, &method_string);
+	if (strcmp (method_string, "file") == 0) {
+		gchar *slashified, *p, *full;
+		GnomeVFSURI *result;
+
+		if (g_ascii_strncasecmp (text_uri, "file://", 7) == 0)
+			rest += 2;
+		slashified = g_strdup (rest);
+		for (p = slashified; *p; p++)
+			if (*p == '\\')
+				*p = '/';
+		if (*slashified == '/')
+			full = g_strconcat ("file://", slashified, NULL);
+		else
+			full = g_strconcat ("file:///", slashified, NULL);
+		result = gnome_vfs_uri_new_private (full, FALSE, FALSE, TRUE);
+		g_free (full);
+		g_free (slashified);
+		g_free (method_string);
+
+		return result;
+	}
+	g_free (method_string);
+#endif
 	return gnome_vfs_uri_new_private (text_uri, FALSE, FALSE, TRUE);
 }
 
@@ -799,6 +840,12 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 				char *tmp = separator - 1;
 				if ((separator != mutable_base_uri) && (*tmp != '/')) {
 					*separator = '\0';
+				} else {
+					/* Maybe there is no domain part and this is a toplevel URI's child */
+					char *tmp2 = strstr (mutable_base_uri, ":///");
+					if (tmp2 != NULL && tmp2 + 3 == separator) {
+						*(separator + 1) = '\0';
+					}
 				}
 			}
 		}
@@ -845,7 +892,12 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
  * @base: The base URI.
  * @relative_reference: A string representing a possibly relative URI reference
  * 
- * Create a new URI from @relative_reference, relative to @base.
+ * Create a new URI from @relative_reference, relative to @base. The resolution
+ * algorithm follows RFC 2396. For details, see section 5.2 of
+ * http://www.ietf.org/rfc/rfc2396.txt .
+ *
+ * In short, if the @base URI ends in '/', @relative_reference is appended to @base,
+ * otherwise it replaces the part of @base after the last '/'.
  *
  * Return value: The new URI.
  **/
@@ -1103,7 +1155,7 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 			 GnomeVFSURIHideOptions hide_options)
 {
 	GString *string;
-	gchar *result;
+	gchar *result, *tmp;
 
 	string = g_string_new (uri->method_string);
 	g_string_append_c (string, ':');
@@ -1126,14 +1178,18 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 
 		if (top_level_uri->user_name != NULL
 			&& (hide_options & GNOME_VFS_URI_HIDE_USER_NAME) == 0) {
-			g_string_append (string, top_level_uri->user_name);
+			tmp = gnome_vfs_escape_string (top_level_uri->user_name);
+			g_string_append (string, tmp);
+			g_free (tmp);
 			shown_user_pass = TRUE;
 		}
 
 		if (top_level_uri->password != NULL
 			&& (hide_options & GNOME_VFS_URI_HIDE_PASSWORD) == 0) {
+			tmp = gnome_vfs_escape_string (top_level_uri->password);
 			g_string_append_c (string, ':');
-			g_string_append (string, top_level_uri->password);
+			g_string_append (string, tmp);
+			g_free (tmp);
 			shown_user_pass = TRUE;
 		}
 
